/*-
 * Copyright (c) 2004 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_signal.c,v 1.1 2004/02/29 00:08:27 nork Exp $
 */

/*
 *
 */
#include "pluginwrapper.h"
#include "linux.h"
#include <errno.h>
#include <stddef.h>
#include <sys/param.h>
#include <sys/signal.h>
#include <sys/signalvar.h>


static int linux_to_bsd_signal[LINUX_SIGTBLSZ] = {
	SIGHUP,  SIGINT,    SIGQUIT, SIGILL,
	SIGTRAP, SIGABRT,   SIGBUS,  SIGFPE,
	SIGKILL, SIGUSR1,   SIGSEGV, SIGUSR2,
	SIGPIPE, SIGALRM,   SIGTERM, SIGBUS,
	SIGCHLD, SIGCONT,   SIGSTOP, SIGTSTP,
	SIGTTIN, SIGTTOU,   SIGURG,  SIGXCPU,
	SIGXFSZ, SIGVTALRM, SIGPROF, SIGWINCH,
	SIGIO,   SIGURG,    SIGSYS
};


static int bsd_to_linux_signal[LINUX_SIGTBLSZ] = {
	LINUX_SIGHUP,  LINUX_SIGINT,    LINUX_SIGQUIT, LINUX_SIGILL,
	LINUX_SIGTRAP, LINUX_SIGABRT,   0,             LINUX_SIGFPE,
	LINUX_SIGKILL, LINUX_SIGBUS,    LINUX_SIGSEGV, LINUX_SIGSYS,
	LINUX_SIGPIPE, LINUX_SIGALRM,   LINUX_SIGTERM, LINUX_SIGURG,
	LINUX_SIGSTOP, LINUX_SIGTSTP,   LINUX_SIGCONT, LINUX_SIGCHLD,
	LINUX_SIGTTIN, LINUX_SIGTTOU,   LINUX_SIGIO,   LINUX_SIGXCPU,
	LINUX_SIGXFSZ, LINUX_SIGVTALRM, LINUX_SIGPROF, LINUX_SIGWINCH,
	0,             LINUX_SIGUSR1,   LINUX_SIGUSR2
};


/*
 * signal(3)
 */
extern sigset_t _sigintr;

void (*
signal( int l_sig, void (*func)(int) )
     )(int)
{
	int sig = linux_to_bsd_signal[l_sig];
	struct sigaction sa, osa;

	sa.sa_handler = func;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	if(  sigismember(&_sigintr, sig) == 0  )
		sa.sa_flags |= SA_RESTART;
	if(  _sigaction(sig, &sa, &osa) < 0  )
		osa.sa_handler = SIG_ERR;

	dprintf("signal(%d(%d on linux), ...) / ERRNO = %d", sig, l_sig, errno);

	return osa.sa_handler;
}
